@@ -15,12 +15,12 @@ XML::Filter::XML_Directory_Pruner - SAX2 filter for restricting the output of th
  my $writer = XML::SAX::Writer->new(Output=>\$output);
  my $pruner = XML::Filter::XML_Directory_Pruner->new(Handler=>$writer);
 
- $pruner->exclude(matching=>"(.*)\\.ph\$");
+ $pruner->exclude(matching=>["(.*)\\.ph\$"]);
  $pruner->include(ending=>[".pm"]);
 
- my $directory = XML::Directory::SAX->new(Handler=>$pruner,
-                                       detail=>2,
-                                       depth=>1);
+ my $directory = XML::Directory::SAX->new(Handler => $pruner,
+                                          detail  => 2,
+                                          depth   => 1);
 
  $directory->parse_dir($INC[0]);
 
@@ -35,12 +35,56 @@ use strict;
 
 use Exporter;
 use XML::SAX::Base;
+use MIME::Types;
 
-$XML::Filter::XML_Directory_Pruner::VERSION   = '1.1';
+$XML::Filter::XML_Directory_Pruner::VERSION   = '1.2';
 @XML::Filter::XML_Directory_Pruner::ISA       = qw (Exporter XML::SAX::Base);
 @XML::Filter::XML_Directory_Pruner::EXPORT    = qw ();
 @XML::Filter::XML_Directory_Pruner::EXPORT_OK = qw ();
 
+my %__typeof = ();
+my $__mtypes = undef;
+
+=head1 PACKAGE METHODS 
+
+=head2 __PACKAGE__->mtype($file)
+
+Return the media type, as defined by the I<MIME::Types> package, associated with I<$file>.
+
+=cut
+
+sub mtype {
+  my $pkg   = shift;
+  my $fname = shift;
+
+  #
+
+  $fname =~ /^(.*)\.([^\.]+)$/;
+  if (! $2) { return undef; }
+
+  if (exists($__typeof{$2})) {
+    return $__typeof{$2};
+  }
+
+  $__mtypes ||= MIME::Types->new()
+    || return undef;
+
+
+  #
+
+  my $mime = $__mtypes->mimeTypeOf($2);
+  
+  if (! $mime) {
+    $__typeof{$2} = undef;
+    return $__typeof{$2};
+  }
+  
+  #
+
+  $__typeof{$2} = $mime->mediaType();
+  return $__typeof{$2};
+}
+
 =head1 OBJECT METHODS
 
 =head2 $pkg = __PACKAGE__->new()
@@ -65,10 +109,12 @@ Array ref.
 
 B<matching>
 
-String. Regular expression.
+Array ref. One or more regular expressions.
 
 I<note that when this expression is compared, leaning toothpicks (e.g. : /$pattern/) are provided for you.>
 
+In earlier releases, only a string was expected. Newer releases are backward compatible.
+
 =item *
 
 B<starting>
@@ -90,11 +136,12 @@ sub include {
     my $args = { @_ };
 
     if (ref($args->{'include'})  eq "ARRAY") {
-      push (@{$self->{__PACKAGE__.'__include'}},@{$args->{'exclude'}});
+      push (@{$self->{__PACKAGE__.'__include'}},@{$args->{'include'}});
     }
 
     if ($args->{'matching'}) {
-      $self->{__PACKAGE__.'__include_matching'} = $args->{'matching'};
+      $self->{__PACKAGE__.'__include_matching'} = (ref($args->{'matching'} eq "ARRAY")) ? 
+	$args->{'matching'} : [$args->{'matching'}];
     }
 
     if (ref($args->{'starting'}) eq "ARRAY") {
@@ -105,6 +152,10 @@ sub include {
 	push (@{$self->{__PACKAGE__.'__include_ending'}},@{$args->{'ending'}});
     }
 
+    if ($args->{'directories'}) {
+      $self->{__PACKAGE__.'__include_subdirs'} = 1;
+    }
+
     return 1;
 }
 
@@ -126,10 +177,12 @@ Array ref.
 
 B<matching>
 
-String. Regular expression.
+Array ref. One or more regular expressions.
 
 I<note that when this expression is compared, leaning toothpicks (e.g. : /$pattern/) are provided for you.>
 
+In earlier releases, only a string was expected. Newer releases are backward compatible.
+
 =item *
 
 B<starting>
@@ -165,7 +218,8 @@ sub exclude {
     }
 
     if ($args->{'matching'}) {
-      $self->{__PACKAGE__.'__exclude_matching'} = $args->{'matching'};
+      $self->{__PACKAGE__.'__exclude_matching'} = (ref($args->{'matching'}) eq "ARRAY") ? 
+	$args->{'matching'} : [ $args->{'matching'}];
     }
 
     if (ref($args->{'starting'}) eq "ARRAY") {
@@ -175,7 +229,7 @@ sub exclude {
     if (ref($args->{'ending'})   eq "ARRAY") {
       push (@{$self->{__PACKAGE__.'__exclude_ending'}},@{$args->{'ending'}});
     }
-    
+
     $self->{__PACKAGE__.'__exclude_subdirs'} = $args->{'directories'};
     $self->{__PACKAGE__.'__exclude_files'}   = $args->{'files'};
     return 1;
@@ -215,7 +269,33 @@ sub skip_level {
   return $_[0]->{__PACKAGE__.'__skip'};
 }
 
-=head2 $pkg->debug($debug)
+=head2 $pkg->debug($int)
+
+Read/write debugging flags.
+
+By default, the package watches and performs actions if the debug level is greater than or equal to :
+
+=over
+
+=item *
+
+I<1>
+
+Nothing.
+
+=item *
+
+I<2>
+
+Prints to STDERR the type, name and level of the current element.
+
+=item *
+
+I<3>
+
+Prints to STDERR the results of checks in $pkg->_compare()
+
+=back
 
 =cut
 
@@ -224,7 +304,7 @@ sub debug {
   my $debug = shift;
 
   if (defined($debug)) {
-    $self->{__PACKAGE__.'__debug'} = ($debug) ? 1 : 0;
+    $self->{__PACKAGE__.'__debug'} = ($debug) ? (int($debug)) ? $debug : 1 : 0;
   }
 
   return $self->{__PACKAGE__.'__debug'};
@@ -237,8 +317,8 @@ sub debug {
 =cut
 
 sub start_element {
-  my $self = shift;
-  my $data = shift;
+  my $self  = shift;
+  my $data  = shift;
 
   $self->on_enter_start_element($data);
   $self->compare($data);
@@ -257,12 +337,18 @@ sub on_enter_start_element {
 
   $self->{__PACKAGE__.'__level'} ++;
 
-  if ($self->debug()) {
+#  if ($data->{Name} =~ /^(directory|file)$/) {
+#    $self->{__PACKAGE__.'__'.$1} ++;
+#    map { print " "; } (0..$self->{__PACKAGE__.'__'.$1});
+#    print $self->{__PACKAGE__.'__'.$1} ." [$1] $data->{Attributes}->{'{}name'}->{Value} ".__PACKAGE__."\n";
+#  }
+
+  if ($self->debug() >= 2) {
     map { print STDERR " "; } (0..$self->current_level);
     print STDERR "[".$self->current_level."] $data->{Name} : ";
     # Because sometimes auto-vivification
     # is not what you want.
-    if (exists($data->{Attributes}->{'{}same'})) {
+    if (exists($data->{Attributes}->{'{}name'})) {
       print STDERR $data->{Attributes}->{'{}name'}->{Value};
     }
 
@@ -284,7 +370,7 @@ sub end_element {
     $self->SUPER::end_element($data);
   }
 
-  $self->on_exit_end_element();
+  $self->on_exit_end_element($data);
   return 1;
 }
 
@@ -294,11 +380,16 @@ sub end_element {
 
 sub on_exit_end_element {
   my $self = shift;
+  my $data = shift;
 
   if ($self->{__PACKAGE__.'__skip'} == $self->{__PACKAGE__.'__level'}) {
     $self->{__PACKAGE__.'__skip'} = 0;
   }
 
+  if ($data->{Name} =~ /^(directory|file)$/) {
+    $self->{__PACKAGE__.'__'.$1} --;
+  }
+
   $self->{__PACKAGE__.'__level'} --;
   return 1;
 }
@@ -326,9 +417,14 @@ sub compare {
   my $self = shift;
   my $data = shift;
 
-  if (($data->{'Name'} =~ /^(file|directory)$/) && (! $self->{__PACKAGE__.'__skip'})) {
-    $self->{__PACKAGE__.'__ima'} = $1;
-    $self->_compare($data->{Attributes}->{'{}name'}->{Value});
+  if ($data->{'Name'} =~ /^(file|directory)$/) {
+    # map { print " "; } (0..$self->{__PACKAGE__.'__'.$1});
+    # print $self->{__PACKAGE__.'__'.$1} ." <$1> $data->{Attributes}->{'{}name'}->{Value} ($self->{__PACKAGE__.'__skip'})\n";
+
+    if (! $self->{__PACKAGE__.'__skip'}) {
+      $self->{__PACKAGE__.'__ima'} = $1;
+      $self->_compare($data->{Attributes}->{'{}name'}->{Value});
+    }
   }
 
   return 1;
@@ -358,76 +454,127 @@ sub _compare {
 
   if ($self->{__PACKAGE__.'__level'} == 2) { return 1; }
 
-  # Include subdirectories?
+  #
 
-  if (($ok) && ($self->{__PACKAGE__.'__ima'} eq "directory" && $self->{__PACKAGE__.'__exclude_subdirs'})) {
-    $ok = 0;
+  if ($self->{__PACKAGE__.'__ima'} eq "directory") {
+    if (($ok) && ($self->{__PACKAGE__.'__exclude_subdirs'})) {
+      print STDERR "10 - EXCLUDING $data BECAUSE I AM A DIRECTORY\n"
+	if ($self->debug() >= 3);
+      $ok = 0;
+    }
   }
 
   if (($ok) && ($self->{__PACKAGE__.'__ima'} eq "file" && $self->{__PACKAGE__.'__exclude_files'})) {
+    print STDERR "20 - EXCLUDING $data BECAUSE I AM A FILE\n"
+      if ($self->debug() >= 3);
     $ok = 0;
   }
 
   #
 
-  if (($ok) && ($self->{__PACKAGE__.'__include_matching'})) {
-    my $pattern = $self->{__PACKAGE__.'__include_matching'};
-    $ok = ($data =~ /$pattern/) ? 1 : 0;
+  if (($ok) && ($self->{__PACKAGE__.'__include_matching'} eq "ARRAY")) {
+    foreach my $pattern (@{$self->{__PACKAGE__.'__include_matching'}}) {
+      $ok = ($data =~ /$pattern/) ? 1 : 0;
+
+      if ($ok) {
+	print STDERR "20 - INCLUDING $data BECAUSE IT MATCHES PATTERN '$pattern'\n"
+	  if ($self->debug() >= 3);
+	last;
+      }
+    }
   }
 
   if (($ok) && (ref($self->{__PACKAGE__.'__include'}) eq "ARRAY")) {
     foreach my $match (@{$self->{__PACKAGE__.'__include'}}) {
       $ok = ($data =~ /^($match)$/) ? 0 : 1;
-      last if (! $ok);
+
+      if ($ok) {
+	print STDERR "30 - INCLUDING $data BECAUSE IT MATCHES '$match'\n"
+	  if ($self->debug() >= 3);
+	last;
+      }
     }
   }
 
   if (($ok) && (ref($self->{__PACKAGE__.'__include_starting'}) eq "ARRAY")) {
     foreach my $match (@{$self->{__PACKAGE__.'__include_starting'}}) {
       $ok = ($data =~ /^($match)(.*)$/) ? 1 : 0;
-      last if (! $ok);
+
+      if ($ok) {
+	print STDERR "40 - INCLUDING $data BECAUSE IT STARTS WITH '$match'\n"
+	  if ($self->debug() >= 3);
+	last;
+      }
     }
   }
 
   if (($ok) && (ref($self->{__PACKAGE__.'__include_ending'}) eq "ARRAY")) {
     foreach my $match (@{$self->{__PACKAGE__.'__include_ending'}}) {
       $ok = ($data =~ /^(.*)($match)$/) ? 1 : 0;
-      last if (! $ok);
+
+      if ($ok) {
+	print STDERR "50 - INCLUDING $data BECAUSE IT ENDS WITH '$match'\n"
+	  if ($self->debug() >= 3);
+	last;
+      }
     }
   }
 
   #
 
-  if (($ok) && ($self->{__PACKAGE__.'__exclude_matching'})) {
-    my $pattern = $self->{__PACKAGE__.'__exclude_matching'};
-    $ok = ($data =~ /$pattern/) ? 0 : 1;
+  if (($ok) && ($self->{__PACKAGE__.'__exclude_matching'} eq "ARRAY")) {
+    foreach  my $pattern (@{$self->{__PACKAGE__.'__exclude_matching'}}) {
+      $ok = ($data =~ /$pattern/) ? 0 : 1;
+
+      if (! $ok) {
+	print STDERR "30 - EXCLUDING $data BECAUSE IT MATCHES PATTERN '$pattern'\n"
+	  if ($self->debug() >= 3);
+	last;
+      }
+    }
   }
 
   if (($ok) && (ref($self->{__PACKAGE__.'__exclude'}) eq "ARRAY")) {
     foreach my $match (@{$self->{__PACKAGE__.'__exclude'}}) {
       $ok = ($data =~ /^($match)$/) ? 0 : 1;
-      last if (! $ok);
+
+      if (! $ok) {
+	print STDERR "40 - EXCLUDING $data BECAUSE IT MATCHES '$match'\n"
+	  if ($self->debug() >= 3);
+	last;
+      }
     }
   }
 
   if (($ok) && (ref($self->{__PACKAGE__.'__exclude_starting'}) eq "ARRAY")) {
     foreach my $match (@{$self->{__PACKAGE__.'__exclude_starting'}}) {
       $ok = ($data =~ /^($match)(.*)$/) ? 0 : 1;
-      last if (! $ok);
+
+      if (! $ok) {
+	print STDERR "50 - EXCLUDING $data BECAUSE IT STARTS WITH '$match'\n"
+	  if ($self->debug() >= 3);
+	last;
+      }
     }
   }
 
   if (($ok) && (ref($self->{__PACKAGE__.'__exclude_ending'}) eq "ARRAY")) {
     foreach my $match (@{$self->{__PACKAGE__.'__exclude_ending'}}) {
       $ok = ($data =~ /^(.*)($match)$/) ? 0 : 1;
-      last if (! $ok);
+
+      if (! $ok) {
+	print STDERR "60 - EXCLUDING $data BECAUSE IT ENDS WITH '$match'\n"
+	  if ($self->debug() >= 3);
+	last;
+      }
     }
   }
 
+  #
+
   if (! $ok) {
-    if ($self->debug()) {
-      print STDERR "SKIPPING '$data' at $self->{__PACKAGE__.'__level'}\n";
-    }
+    print STDERR "SKIPPING '$data' at $self->{__PACKAGE__.'__level'}\n"
+      if ($self->debug() >= 2);
 
     $self->{__PACKAGE__.'__skip'} = $self->{__PACKAGE__.'__level'};
   }
@@ -438,11 +585,11 @@ sub _compare {
 
 =head1 VERSION
 
-1.1
+1.2
 
 =head1 DATE
 
-June 30, 2002
+July 07, 2002
 
 =head1 AUTHOR
 
@@ -454,7 +601,7 @@ Aaron Straup Cope
 
 =item *
 
-Allow for inclusion/exclusion based on mime-type
+Allow for inclusion/exclusion based on MIME and/or media type
 
 =back
 
@@ -464,6 +611,8 @@ L<XML::Directory::SAX>
 
 L<XML::SAX::Base>
 
+L<MIME::Types>
+
 =head1 LICENSE
 
 Copyright (c) 2002, Aaron Straup Cope. All Rights Reserved.
